@@ -9,10 +9,11 @@ from qibo.abstractions.abstract_gates import Gate, Channel, SpecialGate, Paramet
 QASM_GATES = {"h": "H", "x": "X", "y": "Y", "z": "Z",
               "rx": "RX", "ry": "RY", "rz": "RZ",
               "u1": "U1", "u2": "U2", "u3": "U3",
-              "cx": "CNOT", "swap": "SWAP", "cz": "CZ",
+              "cx": "CNOT", "swap": "SWAP", "fswap": "FSWAP", "cz": "CZ",
               "crx": "CRX", "cry": "CRY", "crz": "CRZ",
               "cu1": "CU1", "cu3": "CU3",
-              "ccx": "TOFFOLI", "id": "I"}
+              "ccx": "TOFFOLI", "id": "I", "s": "S",
+              "sdg": "SDG", "t": "T", "tdg": "TDG"}
 PARAMETRIZED_GATES = {"rx", "ry", "rz", "u1", "u2", "u3",
                       "crx", "cry", "crz", "cu1", "cu3"}
 
@@ -162,6 +163,94 @@ class Z(Gate):
         return gate
 
 
+class S(Gate):
+    """The S gate.
+
+    Corresponds to the following unitary matrix
+
+    .. math::
+        \\begin{pmatrix}
+        1 & 0 \\\\
+        0 & i \\\\
+        \\end{pmatrix}
+
+    Args:
+        q (int): the qubit id number.
+    """
+
+    def __init__(self, q):
+        super().__init__()
+        self.name = "s"
+        self.target_qubits = (q,)
+        self.init_args = [q]
+
+
+class SDG(Gate):
+    """The conjugate transpose of the S gate.
+
+    Corresponds to the following unitary matrix
+
+    .. math::
+        \\begin{pmatrix}
+        1 & 0 \\\\
+        0 & -i \\\\
+        \\end{pmatrix}
+
+    Args:
+        q (int): the qubit id number.
+    """
+
+    def __init__(self, q):
+        super().__init__()
+        self.name = "sdg"
+        self.target_qubits = (q,)
+        self.init_args = [q]
+
+
+class T(Gate):
+    """The T gate.
+
+    Corresponds to the following unitary matrix
+
+    .. math::
+        \\begin{pmatrix}
+        1 & 0 \\\\
+        0 & e^{i \\pi / 4} \\\\
+        \\end{pmatrix}
+
+    Args:
+        q (int): the qubit id number.
+    """
+
+    def __init__(self, q):
+        super().__init__()
+        self.name = "t"
+        self.target_qubits = (q,)
+        self.init_args = [q]
+
+
+class TDG(Gate):
+    """The conjugate transpose of the T gate.
+
+    Corresponds to the following unitary matrix
+
+    .. math::
+        \\begin{pmatrix}
+        1 & 0 \\\\
+        0 & e^{-i \\pi / 4} \\\\
+        \\end{pmatrix}
+
+    Args:
+        q (int): the qubit id number.
+    """
+
+    def __init__(self, q):
+        super().__init__()
+        self.name = "tdg"
+        self.target_qubits = (q,)
+        self.init_args = [q]
+
+
 class I(Gate):
     """The identity gate.
 
@@ -893,6 +982,31 @@ class SWAP(Gate):
         self.init_args = [q0, q1]
 
 
+class FSWAP(Gate):
+    """The fermionic swap gate.
+
+    Corresponds to the following unitary matrix
+
+    .. math::
+        \\begin{pmatrix}
+        1 & 0 & 0 & 0 \\\\
+        0 & 0 & 1 & 0 \\\\
+        0 & 1 & 0 & 0 \\\\
+        0 & 0 & 0 & -1 \\\\
+        \\end{pmatrix}
+
+    Args:
+        q0 (int): the first qubit to be f-swapped id number.
+        q1 (int): the second qubit to be f-swapped id number.
+    """
+
+    def __init__(self, q0, q1):
+        super(FSWAP, self).__init__()
+        self.name = "fswap"
+        self.target_qubits = (q0, q1)
+        self.init_args = [q0, q1]
+
+
 class fSim(ParametrizedGate):
     """The fSim gate defined in `arXiv:2001.08343 <https://arxiv.org/abs/2001.08343>`_.
 
@@ -1107,7 +1221,7 @@ class VariationalLayer(ParametrizedGate):
             If ``None`` the name ``"VariationalLayer"`` will be used.
 
     Example:
-        ::
+        .. testcode::
 
             import numpy as np
             from qibo.models import Circuit
@@ -1267,7 +1381,7 @@ class KrausChannel(Channel):
           the corresponding matrix as a ``np.ndarray`` or ``tf.Tensor``.
 
     Example:
-        ::
+        .. testcode::
 
             import numpy as np
             from qibo.models import Circuit
