@@ -8,6 +8,21 @@ class NumpyBackend(abstract.AbstractBackend):
     description = "Uses `np.einsum` to apply gates to states via matrix " \
                   "multiplication."
 
+    TEST_REGRESSIONS = {
+        "test_measurementresult_apply_bitflips": [
+                [0, 0, 0, 0, 2, 3, 0, 0, 0, 0],
+                [0, 0, 0, 0, 2, 3, 0, 0, 0, 0],
+                [0, 0, 0, 0, 0, 1, 0, 0, 0, 0],
+                [0, 0, 0, 0, 2, 0, 0, 0, 0, 0]
+            ],
+        "test_probabilistic_measurement": {0: 249, 1: 231, 2: 253, 3: 267},
+        "test_unbalanced_probabilistic_measurement": {0: 171, 1: 148, 2: 161, 3: 520},
+        "test_post_measurement_bitflips_on_circuit": [
+                {5: 30}, {5: 18, 4: 5, 7: 4, 1: 2, 6: 1},
+                {4: 8, 2: 6, 5: 5, 1: 3, 3: 3, 6: 2, 7: 2, 0: 1}
+            ],
+    }
+
     def __init__(self):
         super().__init__()
         import numpy as np
@@ -38,6 +53,12 @@ class NumpyBackend(abstract.AbstractBackend):
                     "Cannot change the number of threads.")
         abstract.AbstractBackend.set_threads(self, nthreads)
 
+    def set_platform(self, platform):
+        pass
+
+    def get_platform(self):
+        return None
+
     def to_numpy(self, x):
         return x
 
@@ -48,7 +69,7 @@ class NumpyBackend(abstract.AbstractBackend):
         if isinstance(dtype, str):
             dtype = self.dtypes(dtype)
         if isinstance(x, self.backend.ndarray):
-            return x.astype(dtype)
+            return x.astype(dtype, copy=False)
         return self.backend.array(x, dtype=dtype)
 
     def diag(self, x, dtype='DTYPECPX'):
@@ -390,26 +411,21 @@ class NumpyBackend(abstract.AbstractBackend):
     def on_cpu(self): # pragma: no cover
         return self.device()
 
-    def cpu_tensor(self, x, dtype=None):
+    def cpu_tensor(self, x, dtype=None): # pragma: no cover
         if dtype is None:
             dtype = x.dtype
         return self.np.asarray(x, dtype=dtype)
 
-    def cpu_cast(self, x, dtype='DTYPECPX'):
+    def cpu_cast(self, x, dtype='DTYPECPX'): # pragma: no cover
         dtype = self._dtypes.get(dtype)
         return self.np.array(x, dtype=dtype)
 
-    def cpu_assign(self, state, i, piece):
+    def cpu_assign(self, state, i, piece): # pragma: no cover
         state.pieces[i] = self.to_numpy(piece)
 
     def transpose_state(self, pieces, state, nqubits, order):
         pieces = self.reshape(self.backend.stack(pieces), nqubits * (2,))
         return self.reshape(self.transpose(pieces, order), state.shape)
 
-    def swap_pieces(self, piece0, piece1, new_global, nlocal): # pragma: no cover
-        raise_error(NotImplementedError,
-                    "Swap pieces is not implemented for {} backend as it "
-                    "does not support multigpu.".format(self.name))
-
     def assert_allclose(self, value, target, rtol=1e-7, atol=0.0):
         self.np.testing.assert_allclose(value, target, rtol=rtol, atol=atol)
