@@ -76,12 +76,15 @@ def _executor(params): # pragma: no cover
     return ParallelResources().run(params)
 
 
-def parallel_execution(circuit, states, processes=None):
+def parallel_execution(circuit, states, processes=None):  # pragma: no cover
     """Execute circuit for multiple states.
 
     Example:
-        ::
+        .. testcode::
 
+            import qibo
+            original_backend = qibo.get_backend()
+            qibo.set_backend("qibotf")
             from qibo import models, set_threads
             from qibo.parallel import parallel_execution
             import numpy as np
@@ -94,6 +97,7 @@ def parallel_execution(circuit, states, processes=None):
             set_threads(1)
             # execute in parallel
             results = parallel_execution(circuit, states, processes=2)
+            qibo.set_backend(original_backend)
 
     Args:
         circuit (qibo.models.Circuit): the input circuit.
@@ -124,12 +128,15 @@ def parallel_execution(circuit, states, processes=None):
     return results
 
 
-def parallel_parametrized_execution(circuit, parameters, initial_state=None, processes=None):
+def parallel_parametrized_execution(circuit, parameters, initial_state=None, processes=None):  # pragma: no cover
     """Execute circuit for multiple parameters and fixed initial_state.
 
     Example:
-        ::
+        .. testcode::
 
+            import qibo
+            original_backend = qibo.get_backend()
+            qibo.set_backend("qibotf")
             from qibo import models, gates, set_threads
             from qibo.parallel import parallel_parametrized_execution
             import numpy as np
@@ -151,6 +158,7 @@ def parallel_parametrized_execution(circuit, parameters, initial_state=None, pro
             set_threads(1)
             # execute in parallel
             results = parallel_parametrized_execution(circuit, parameters, processes=2)
+            qibo.set_backend(original_backend)
 
     Args:
         circuit (qibo.models.Circuit): the input circuit.
@@ -183,7 +191,7 @@ def parallel_parametrized_execution(circuit, parameters, initial_state=None, pro
     return results
 
 
-def _check_parallel_configuration(processes):
+def _check_parallel_configuration(processes):  # pragma: no cover
     """Check if configuration is suitable for efficient parallel execution."""
     import sys, psutil
     from qibo import get_device, get_backend, get_threads
