@@ -65,6 +65,32 @@ def test_z(backend):
     K.assert_allclose(final_state, target_state)
 
 
+def test_s(backend):
+    final_state = apply_gates([gates.H(0), gates.H(1), gates.S(1)], nqubits=2)
+    target_state = np.array([0.5, 0.5j, 0.5, 0.5j])
+    K.assert_allclose(final_state, target_state)
+
+
+def test_sdg(backend):
+    final_state = apply_gates([gates.H(0), gates.H(1), gates.SDG(1)], nqubits=2)
+    target_state = np.array([0.5, -0.5j, 0.5, -0.5j])
+    K.assert_allclose(final_state, target_state)
+
+
+def test_t(backend):
+    final_state = apply_gates([gates.H(0), gates.H(1), gates.T(1)], nqubits=2)
+    target_state = np.array([0.5, (1 + 1j) / np.sqrt(8),
+                             0.5, (1 + 1j) / np.sqrt(8)])
+    K.assert_allclose(final_state, target_state)
+
+
+def test_tdg(backend):
+    final_state = apply_gates([gates.H(0), gates.H(1), gates.TDG(1)], nqubits=2)
+    target_state = np.array([0.5, (1 - 1j) / np.sqrt(8),
+                             0.5, (1 - 1j) / np.sqrt(8)])
+    K.assert_allclose(final_state, target_state)
+
+
 def test_identity(backend):
     gatelist = [gates.H(0), gates.H(1), gates.I(0), gates.I(1)]
     final_state = apply_gates(gatelist, nqubits=2)
@@ -207,6 +233,14 @@ def test_swap(backend):
     K.assert_allclose(final_state, target_state)
 
 
+def test_fswap(backend):
+    final_state = apply_gates([gates.H(0), gates.X(1), gates.FSWAP(0, 1)], nqubits=2)
+    target_state = np.zeros_like(final_state)
+    target_state[2] = 1.0 / np.sqrt(2)
+    target_state[3] = -1.0 / np.sqrt(2)
+    K.assert_allclose(final_state, target_state)
+
+
 def test_multiple_swap(backend):
     gatelist = [gates.X(0), gates.X(2), gates.SWAP(0, 1), gates.SWAP(2, 3)]
     final_state = apply_gates(gatelist, nqubits=4)
@@ -291,9 +325,6 @@ def test_unitary_initialization(backend):
         gate = gates.Unitary(matrix, 0, 1)
     with pytest.raises(TypeError):
         gate = gates.Unitary("abc", 0, 1)
-    if K.op is not None:
-        with pytest.raises(NotImplementedError):
-            gate = gates.Unitary(matrix, 0, 1, 2)
 
 
 def test_unitary_common_gates(backend):
@@ -320,6 +351,28 @@ def test_unitary_common_gates(backend):
     K.assert_allclose(final_state, target_state)
 
 
+def test_unitary_multiqubit(backend):
+    gatelist = [gates.H(i) for i in range(4)]
+    gatelist.append(gates.CNOT(0, 1))
+    gatelist.append(gates.CNOT(2, 3))
+    gatelist.extend(gates.X(i) for i in range(4))
+
+    h = np.array([[1, 1], [1, -1]]) / np.sqrt(2)
+    x = np.array([[0, 1], [1, 0]])
+    cnot = np.array([[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 0, 1], [0, 0, 1, 0]])
+    matrix = np.kron(np.kron(x, x), np.kron(x, x))
+    matrix = matrix @ np.kron(cnot, cnot)
+    matrix = matrix @ np.kron(np.kron(h, h), np.kron(h, h))
+    unitary = gates.Unitary(matrix, 0, 1, 2, 3)
+    if K.name == "qibotf":
+        with pytest.raises(NotImplementedError):
+            final_state = apply_gates([unitary], nqubits=4)
+    else:
+        final_state = apply_gates([unitary], nqubits=4)
+        target_state = apply_gates(gatelist, nqubits=4)
+        K.assert_allclose(final_state, target_state)
+
+
 @pytest.mark.parametrize("nqubits", [5, 6])
 def test_variational_layer(backend, nqubits):
     theta = 2 * np.pi * np.random.random(nqubits)
@@ -527,7 +580,7 @@ def test_thermal_relaxation_channel_errors(backend, t1, t2, time, excpop):
 def test_fused_gate_init(backend):
     gate = gates.FusedGate(0)
     gate = gates.FusedGate(0, 1)
-    if K.op is not None:
+    if K.is_custom:
         with pytest.raises(NotImplementedError):
             gate = gates.FusedGate(0, 1, 2)
 
